<?php
/**
 * Copyright (c) Xerox Corporation, Codendi Team, 2001-2009. All rights reserved
 *
 * This file is a part of Codendi.
 *
 * Codendi is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version.
 *
 * Codendi is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with Codendi. If not, see <http://www.gnu.org/licenses/>.
 *
 * Portions Copyright 2010 (c) Mélanie Le Bail
 */

require_once 'common/event/Event.class.php';
require_once 'mailman/include/MailmanListDao.class.php';
require_once 'common/dao/CodendiDataAccess.class.php';
require_once 'mailman/include/MailmanList.class.php';
require_once 'plugins_utils.php';
require_once 'plugins/mailman/config.php';

class BackendMailmanList {

    protected $_mailinglistdao = null;

    /**
     * Hold an instance of the class
     */
    protected static $_instance;

    /**
     * Backends are singletons
     */
    public static function instance() {

        if (!isset(self::$_instance)) {
            $c = __CLASS__;
            self::$_instance = new $c;
        }
        return self::$_instance;
    }

    /**
     * @return MailingListDao
     */
    protected function _getMailingListDao() {
        if (!$this->_mailinglistdao) {
          $this->_mailinglistdao = new MailmanListDao(CodendiDataAccess::instance());
        }
        return $this->_mailinglistdao;
    }

    
     /**
     * Update mailman configuration for the given created list
     * Write configuration in temporary file, and load it with mailman config_list tool
     * @return true on success, false otherwise
     */
    protected function createListConfig($listname,$group_list_id,$description,$is_public) {
        // write configuration in temporary file
        $config_file=forge_get_config('tmp_dir')."/mailman_create_".$group_list_id.".in";

        if ($fp = fopen($config_file, 'w')) {
            // Define encoding of this file for Python. See SR #764
            // Please note that this allows config_list to run with UTF-8 strings, but if the
            // description contains non-ascii chars, they will be displayed badly in mailman config web page.
            // Setup the description
            fwrite($fp, "description = '".addslashes($description)."'\n");
            // Approve subscribing
            fwrite($fp, "subscribe_policy = 2\n");
            // Allow up to 500 kB messages
            fwrite($fp, "max_message_size = 500\n");
            // Public archives
            fwrite($fp, "archive_private = False\n");
            // No max number of recipients for a posting
            fwrite($fp, "max_num_recipients = 0\n");

            if ($is_public == MAIL__MAILING_LIST_IS_PRIVATE || $is_public == MAIL__MAILING_LIST_IS_PRIVATE_NOT_MODERATE) { // Private lists
                // Don't advertise this list when people ask what lists are on this machine
                fwrite($fp, "advertised = False\n");
				## Reject mails sent by non-members
	            fwrite($fp, "generic_nonmember_action = 2\n");
				## Do not forward auto discard message
	            fwrite($fp, "forward_auto_discards = 0\n");
            } else {
                // Advertise this list when people ask what lists are on this machine
	            fwrite($fp, "advertised = True\n");
				## Hold mails sent by non-members
	            fwrite($fp, "generic_nonmember_action = 1\n");
				## Forward auto discard message
	            fwrite($fp, "forward_auto_discards = 1\n");
            }
            fclose($fp);

            if (system(forge_get_config('mailman_path')."/bin/config_list -i $config_file $listname") !== false) {
                if (unlink($config_file)) {
					return true;
                }
            }

        }
        return false;
    }
   
    /**
     * Update mailman configuration for the given list
     * Write configuration in temporary file, and load it with mailman config_list tool
     * @return true on success, false otherwise
     */
    protected function updateListConfig($listname,$group_list_id,$description,$password) {
		
        // write configuration in temporary file
        $config_file=forge_get_config('tmp_dir')."/mailman_update_".$group_list_id.".in";

        if ($fp = fopen($config_file, 'w')) {
            // Define encoding of this file for Python. See SR #764
            // Please note that this allows config_list to run with UTF-8 strings, but if the
            // description contains non-ascii chars, they will be displayed badly in mailman config web page.
            // Setup the description
            fwrite($fp, "description = '".addslashes($description)."'\n");
            fclose($fp);

            if (system(forge_get_config('mailman_path')."/bin/config_list -i $config_file $listname") !== false) {
                if (unlink($config_file)) {
					return true;
                }
            }

        }
        return false;
    }
    
    /**
     * Create new mailing list with mailman 'newlist' tool
     * then update the list configuration according to list settings
     * @return true on success, false otherwise
     */
    public function createList($group_list_id) {

        $dar = $this->_getMailingListDao()->searchByGroupListId($group_list_id);

        if ($row = $dar->getRow()) {
        	$user=UserManager::instance()->getUserByID($row['list_admin']);
	    	$list_admin_email = $user ?  $user->getEmail() : "";
	    	$list_name = $row['list_name'];
	    	$list_password = $row['password'];
	    	$list_description = $row['description'];
	    	$list_is_public = $row['is_public'];
            $list_dir = forge_get_config('install_mailman')."/lists/".$list_name;

		    if($list_is_public != MAIL__MAILING_LIST_IS_DELETED) {
			    if (!file_exists($list_dir)) {
				    // Create list
				    system(forge_get_config('mailman_path')."/bin/newlist -q ".$list_name." ".$list_admin_email." ".$list_password." >/dev/null");
				    // Then update configuration
				    if ( is_dir($list_dir) && $this->createListConfig($list_name,$group_list_id,$list_description,$list_is_public) != false ) {
					    $result = $this->_getMailingListDao() -> updateList($group_list_id,$row['group_id'], $list_description, $list_is_public, $list_password, MAIL__MAILING_LIST_IS_CONFIGURED);
					    if (!$result) {
						    printf('Unable to update the list status: '.db_error());
						    return false;
					    }
					    else {
						    return true;
					    }
				    }
				    else {
					    return false;
				    }
			    }
			    else {
					$result = $this->_getMailingListDao() -> updateList($group_list_id,$row['group_id'], $list_description, $list_is_public, $list_password, MAIL__MAILING_LIST_IS_CONFIGURED);
				    if (!$result) {
					    printf('Unable to update the list status: '.db_error());
					    return false;
				    }
				    else {
					    return true;
				    }
			    }
		    }
		}
		return false;
    }

     
    /**
     * Update the list configuration according to list settings
     * @return true on success, false otherwise
     */
    public function updateList($group_list_id) {

        $dar = $this->_getMailingListDao()->searchByGroupListId($group_list_id);

        if ($row = $dar->getRow()) {
        	$user=UserManager::instance()->getUserByID($row['list_admin']);
	    	$list_admin_email = $user ?  $user->getEmail() : "";
	    	$group_id = $row['group_id'];
	    	$list_name = $row['list_name'];
	    	$list_password = $row['password'];
	    	$list_description = $row['description'];
	    	$list_is_public = $row['is_public'];
            $list_dir = forge_get_config('install_mailman')."/lists/".$list_name;

		    if ( is_dir($list_dir) && $list_is_public != MAIL__MAILING_LIST_IS_DELETED ) {
		    	// update configuraion
			    if ( $this->updateListConfig($list_name,$group_list_id,$list_description,$list_password) != false ) {
				    $result = $this->_getMailingListDao()->updateList($group_list_id, $group_id, $list_description, $list_is_public, $list_password, MAIL__MAILING_LIST_IS_CONFIGURED);
				    if (!$result) {
					    printf('Unable to update the list status: '.db_error());
					    return false;
				    }
				    else {
					    return true;
				    }
			    }
			    else {
				    return false;
			    }
		    }
		    else {
				return false;
		    }
		}
		return false;
    }
    
    /**
     * Delete mailing list
     * @return true on success, false otherwise
     */
    public function deleteList($list_name) {
	    $list_dir = forge_get_config('install_mailman')."/lists/".$list_name;
	    if ( is_dir($list_dir) ) {

	    	// Delete the mailing list if asked to and the mailing exists
	    	system(forge_get_config('mailman_path'). '/bin/rmlist -a ' . $list_name .' >/dev/null');
	  		$success = false;
    		if (!file_exists(forge_get_config('install_mailman')."/lists/$list_name")) {
        		$success = true;
   			}
		    if ($success) {
			    $result = $this->_getMailingListDao()->deleteAllSubscribers($list_name); 
		    	if (!$result) {
				    printf('Unable to delete the members of the list: '.db_error());
				    return false;
			    }
	    		$result = $this->_getMailingListDao()->updateDeletedList($list_name);
	    		if (!$result) {
				    printf('Unable to update the deleted list: '.db_error());
				    return false;
	    		}
	    		return true;
		    }
	    	return false;
	    }
    	return false;
    }
           
    /**
     * Synchronise data between the forge and mailman for all active mailing lists
     * @return true on success, false otherwise
     */
    public function configDataAllActiveML() {
    	$dar = $this->_getMailingListDao()->searchAllActiveML();
    
		if (is_dir(forge_get_config('mailman_path'))) {
			$path_to_mailman = forge_get_config('mailman_path');
		} else {
			printf('path_to_mailman path is not set right for this script!!');
			return false;
		
		}
		if (!is_dir(forge_get_config('data_path').'/dumps')) {
			mkdir(forge_get_config('data_path').'/dumps', 0755, true);
		}
		$h1 = fopen(forge_get_config('data_path').'/dumps/mailman-aliases', "w");
    	
        while ($row =& $dar->getRow())
        {
        	$list_name = $row['list_name'];
        	$group_id = $row['group_id']; 

        	//
        	// Update members of the list containing all members of the projet
        	//
			$Group = new Group($group_id);
	        $group_name = $Group->getUnixName();
	        if (strCmp($list_name,$group_name) == 0) {
	        	$this->_getMailingListDao()->deleteAllSubscribers($list_name);
	        	$users = $Group->getUsers(true);
				foreach ($users as $user)
			    {
			        $mail = $user->getEmail();
					$passwd= $user->getUserPw();
					$name= $user->getRealName();
					$member = $this->_getMailingListDao()->SearchUserByEmail($mail,$list_name);
					$arr_member = $member->getRow();
					if (strCmp($arr_member['name'],'') == 0) {
						$res = $this->_getMailingListDao()->newSubscriber($mail,$name,$passwd,$list_name,'N');
						if (!$res) {
							return false;
						}
					}
			    }
	        }

            $list_dir = forge_get_config('install_mailman')."/lists/".$list_name;	
		    if (is_dir($list_dir)) {	

	        	// Set administration password
	        	$password = $row['password'];
				system(forge_get_config('mailman_path')."/bin/change_pw -q -l $list_name -p $password >/dev/null");
		    	
		    	//
	        	// Synchronise the owners of the list and the managers of the projet
	        	//
	        	$managers = $Group->getAdmins();
	        	
	        	$managers_file=$GLOBALS['tmp_dir']."/mailman_managers_".$list_name;
				if ($fp = fopen($managers_file, 'w')) {
					fwrite($fp, "owner='");
					foreach ($managers as $u) {
						$userMail = $u->getEMail();
						fwrite($fp, $userMail."\\n");
					}
					fwrite($fp, "'\n");
	            	fclose($fp);
				}
			    if (system(forge_get_config('mailman_path')."/bin/config_list -i $managers_file $list_name") !== false) {
	                unlink($managers_file);
			    } else {
					return false;
	            }
		    	
	        	//
	        	// Update members of a list
	        	//
		    	$members = $this->_getMailingListDao()->listSubscribers($list_name);
	    	
		    	$members_file=$GLOBALS['tmp_dir']."/mailman_members_".$list_name;
				if ($fp = fopen($members_file, 'w')) {
					for ($i=0; $i<$members->rowCount(); $i++) {
						if ($result = $members->getRow()) { 
							fwrite($fp, $result['address']);
							fwrite($fp, "\n");
						}
			    	}
	            	fclose($fp);
	            	
	            	if (system(forge_get_config('mailman_path'). '/bin/sync_members -f '. $members_file . ' '. $list_name .' >/dev/null') !== false) {
	            		unlink($members_file);
	            	}
				}	
			} else {
				printf($list_name.' does not exist.');
				return false;
			}
			
			//
			// create the lists in a /var/lib/gforge/dumps/mailman-aliases file
			//
			
			if(file_exists($path_to_mailman.'/mail/mailman')) {
			// Mailman 2.1
				$list_str =
$list_name.':              "|'.$path_to_mailman.'/mail/mailman post '.$list_name.'"'."\n"
.$list_name.'-admin:        "|'.$path_to_mailman.'/mail/mailman admin '.$list_name.'"'."\n"
.$list_name.'-bounces:      "|'.$path_to_mailman.'/mail/mailman bounces '.$list_name.'"'."\n"
.$list_name.'-confirm:      "|'.$path_to_mailman.'/mail/mailman confirm '.$list_name.'"'."\n"
.$list_name.'-join:         "|'.$path_to_mailman.'/mail/mailman join '.$list_name.'"'."\n"
.$list_name.'-leave:        "|'.$path_to_mailman.'/mail/mailman leave '.$list_name.'"'."\n"
.$list_name.'-owner:        "|'.$path_to_mailman.'/mail/mailman owner '.$list_name.'"'."\n"
.$list_name.'-request:      "|'.$path_to_mailman.'/mail/mailman request '.$list_name.'"'."\n"
.$list_name.'-subscribe:    "|'.$path_to_mailman.'/mail/mailman subscribe '.$list_name.'"'."\n"
.$list_name.'-unsubscribe:  "|'.$path_to_mailman.'/mail/mailman unsubscribe '.$list_name.'"'."\n\n"
;
			} else {
			// Mailman < 2.1
				$list_str =
$list_name.':		"|'.$path_to_mailman.'/mail/wrapper post '.$list_name.'"'."\n"
.$list_name.'-admin:	"|'.$path_to_mailman.'/mail/wrapper mailowner '.$list_name.'"'."\n"
.$list_name.'-request:	"|'.$path_to_mailman.'/mail/wrapper mailcmd '.$list_name.'"'."\n"
.$list_name.'-owner:	'.$list_name.'-admin'."\n\n";
			}
			fwrite($h1, $list_str);
        }
        
        fclose($h1);
        
        return true;
    }
               
    /**
     * Set to false the private_roster parameter for all active lists
     * @return true on success, false otherwise
     */
    public function configRosterAllActiveML() {
    	$dar = $this->_getMailingListDao()->searchAllActiveML();

        while ($row =& $dar->getRow())
        {
	    	$list_name = $row['list_name'];
	    	$group_list_id = $row['group_list_id'];
		    $list_description = $row['description'];
		    $list_is_public = $row['is_public'];
	        $list_dir = forge_get_config('install_mailman')."/lists/".$list_name;
	        if ( is_dir($list_dir) && $list_is_public != MAIL__MAILING_LIST_IS_DELETED ) {
		    	// update configuraion
			    if ($this->updateListRoster($list_name,$group_list_id,$list_description,$list_is_public) == false) {
			    	return false;
			    }
			}	   
        }
        return true;
    }

    /**
     * Update mailman configuration for the given list
     * Write configuration in temporary file, and load it with mailman config_list tool
     * @return true on success, false otherwise
     */
    protected function updateListRoster($listname,$group_list_id,$description,$is_public) {
        // write configuration in temporary file
        $config_file=forge_get_config('tmp_dir')."/mailman_config_roster_".$group_list_id.".in";

        if ($fp = fopen($config_file, 'w')) {
            // Anyone can see the list of members
            fwrite($fp, "private_roster = False\n");

            fclose($fp);

            if (system(forge_get_config('mailman_path')."/bin/config_list -i $config_file $listname") !== false) {
                if (unlink($config_file)) {
					return true;
                }
            }
        }
        return false;
    }
         
    /**
     * Set configuration for all mailing lists
     * @return true on success, false otherwise
     */
    public function setArchivesToPublic() {
    	$dar = $this->_getMailingListDao()->searchAllActiveML();

        while ($row =& $dar->getRow())
        {
        	$listname = $row['list_name'];
        	$list_is_public = $row['is_public'];
	        
		    $list_dir = forge_get_config('install_mailman')."/lists/".$listname;
		    if ( is_dir($list_dir) && $list_is_public != MAIL__MAILING_LIST_IS_DELETED ) {
		        // write configuration in temporary file
		        $config_file=forge_get_config('tmp_dir')."/mailman_archive_".$listname.".in";
		
		        if ($fp = fopen($config_file, 'w')) {
		            // Public archives
		            fwrite($fp, "archive_private = False\n");
		            fclose($fp);
		
		            if (system(forge_get_config('mailman_path')."/bin/config_list -i $config_file $listname") !== false) {
		                unlink($config_file);
		            } else {
		            	return false;
		            }
		
		        }
			} else {
				return false;
			}
		}
		return true;
		
    }
    
}

?>
